package nns

import (
	client "git.frostfs.info/TrueCloudLab/frostfs-contract/rpcclient/nns"
	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-adm/internal/modules/morph/constants"
	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-adm/internal/modules/morph/helper"
	commonCmd "git.frostfs.info/TrueCloudLab/frostfs-node/cmd/internal/common"
	"github.com/nspcc-dev/neo-go/pkg/rpcclient/invoker"
	"github.com/nspcc-dev/neo-go/pkg/rpcclient/management"
	"github.com/spf13/cobra"
	"github.com/spf13/viper"
)

func nnsWriter(cmd *cobra.Command) (*client.Contract, *helper.LocalActor) {
	v := viper.GetViper()
	c, err := helper.GetN3Client(v)
	commonCmd.ExitOnErr(cmd, "unable to create NEO rpc client: %w", err)

	ac, err := helper.NewLocalActor(cmd, c, constants.CommitteeAccountName)
	commonCmd.ExitOnErr(cmd, "can't create actor: %w", err)

	r := management.NewReader(ac.Invoker)
	nnsCs, err := helper.GetContractByID(r, 1)
	commonCmd.ExitOnErr(cmd, "can't get NNS contract state: %w", err)
	return client.New(ac, nnsCs.Hash), ac
}

func nnsReader(cmd *cobra.Command) (*client.ContractReader, *invoker.Invoker) {
	c, err := helper.GetN3Client(viper.GetViper())
	commonCmd.ExitOnErr(cmd, "unable to create NEO rpc client: %w", err)

	inv := invoker.New(c, nil)
	r := management.NewReader(inv)
	nnsCs, err := helper.GetContractByID(r, 1)
	commonCmd.ExitOnErr(cmd, "can't get NNS contract state: %w", err)

	return client.NewReader(inv, nnsCs.Hash), inv
}
