package buckets

import (
	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-lens/internal/schema/common"
	cid "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/container/id"
)

func (b *PrefixBucket) Filter(typ string, _ any) common.FilterResult {
	switch typ {
	case "cid":
		return b.resolvers.cidResolver(false)
	case "oid":
		return b.resolvers.oidResolver(false)
	default:
		return common.No
	}
}

func (b *PrefixContainerBucket) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "cid":
		id := val.(cid.ID)
		return b.resolvers.cidResolver(b.id.Equals(id))
	case "oid":
		return b.resolvers.oidResolver(false)
	default:
		return common.No
	}
}

func (b *UserBucket) Filter(typ string, _ any) common.FilterResult {
	switch typ {
	case "cid":
		return b.resolvers.cidResolver(false)
	case "oid":
		return b.resolvers.oidResolver(false)
	default:
		return common.No
	}
}

func (b *ContainerBucket) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "cid":
		id := val.(cid.ID)
		return b.resolvers.cidResolver(b.id.Equals(id))
	case "oid":
		return b.resolvers.oidResolver(false)
	default:
		return common.No
	}
}

func (b *UserAttributeKeyBucket) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "cid":
		id := val.(cid.ID)
		return common.IfThenElse(b.id.Equals(id), common.Yes, common.No)
	case "oid":
		return common.Maybe
	case "key":
		key := val.(string)
		return common.IfThenElse(b.key == key, common.Yes, common.No)
	case "value":
		return common.Maybe
	default:
		return common.No
	}
}

func (b *UserAttributeValueBucket) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "oid":
		return common.Maybe
	case "value":
		value := val.(string)
		return common.IfThenElse(b.value == value, common.Yes, common.No)
	default:
		return common.No
	}
}
