package buckets

import (
	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-lens/internal/schema/metabase/records"
)

var (
	GraveyardParser = NewPrefixBucketParser(Graveyard, records.GraveyardRecordParser, Resolvers{
		cidResolver: LenientResolver,
		oidResolver: LenientResolver,
	})

	GarbageParser = NewPrefixBucketParser(Garbage, records.GarbageRecordParser, Resolvers{
		cidResolver: LenientResolver,
		oidResolver: LenientResolver,
	})

	ContainerVolumeParser = NewPrefixBucketParser(ContainerVolume, records.ContainerVolumeRecordParser, Resolvers{
		cidResolver: LenientResolver,
		oidResolver: StrictResolver,
	})

	LockedParser = NewPrefixBucketParser(
		Locked,
		NewContainerBucketParser(
			records.LockedRecordParser,
			Resolvers{
				cidResolver: StrictResolver,
				oidResolver: LenientResolver,
			},
		),
		Resolvers{
			cidResolver: LenientResolver,
			oidResolver: LenientResolver,
		},
	)

	ShardInfoParser = NewPrefixBucketParser(ShardInfo, records.ShardInfoRecordParser, Resolvers{
		cidResolver: StrictResolver,
		oidResolver: StrictResolver,
	})

	PrimaryParser = NewPrefixContainerBucketParser(Primary, records.ObjectRecordParser, Resolvers{
		cidResolver: StrictResolver,
		oidResolver: LenientResolver,
	})

	LockersParser = NewPrefixContainerBucketParser(Lockers, records.ObjectRecordParser, Resolvers{
		cidResolver: StrictResolver,
		oidResolver: LenientResolver,
	})

	TombstoneParser = NewPrefixContainerBucketParser(Tombstone, records.ObjectRecordParser, Resolvers{
		cidResolver: StrictResolver,
		oidResolver: LenientResolver,
	})

	SmallParser = NewPrefixContainerBucketParser(Small, records.SmallRecordParser, Resolvers{
		cidResolver: StrictResolver,
		oidResolver: LenientResolver,
	})

	RootParser = NewPrefixContainerBucketParser(Root, records.RootRecordParser, Resolvers{
		cidResolver: StrictResolver,
		oidResolver: LenientResolver,
	})

	OwnerParser = NewPrefixContainerBucketParser(
		Owner,
		NewUserBucketParser(
			records.OwnerRecordParser,
			Resolvers{
				cidResolver: StrictResolver,
				oidResolver: LenientResolver,
			},
		),
		Resolvers{
			cidResolver: StrictResolver,
			oidResolver: LenientResolver,
		},
	)

	UserAttributeParser = NewUserAttributeKeyBucketParser(
		NewUserAttributeValueBucketParser(records.UserAttributeRecordParser),
	)

	PayloadHashParser = NewPrefixContainerBucketParser(PayloadHash, records.PayloadHashRecordParser, Resolvers{
		cidResolver: StrictResolver,
		oidResolver: StrictResolver,
	})

	ParentParser = NewPrefixContainerBucketParser(Parent, records.ParentRecordParser, Resolvers{
		cidResolver: StrictResolver,
		oidResolver: LenientResolver,
	})

	SplitParser = NewPrefixContainerBucketParser(Split, records.SplitRecordParser, Resolvers{
		cidResolver: StrictResolver,
		oidResolver: StrictResolver,
	})

	ContainerCountersParser = NewPrefixBucketParser(ContainerCounters, records.ContainerCountersRecordParser, Resolvers{
		cidResolver: LenientResolver,
		oidResolver: StrictResolver,
	})

	ECInfoParser = NewPrefixContainerBucketParser(ECInfo, records.ECInfoRecordParser, Resolvers{
		cidResolver: StrictResolver,
		oidResolver: LenientResolver,
	})
)
