package records

import (
	"fmt"

	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-lens/internal/schema/common"
	"github.com/gdamore/tcell/v2"
	"github.com/rivo/tview"
)

func (r *GraveyardRecord) String() string {
	return fmt.Sprintf(
		"Object CID %s OID %s %c Tombstone CID %s OID %s",
		common.FormatSimple(fmt.Sprintf("%-44s", r.object.Container()), tcell.ColorAqua),
		common.FormatSimple(fmt.Sprintf("%-44s", r.object.Object()), tcell.ColorAqua),
		tview.Borders.Vertical,
		common.FormatSimple(fmt.Sprintf("%-44s", r.tombstone.Container()), tcell.ColorAqua),
		common.FormatSimple(fmt.Sprintf("%-44s", r.tombstone.Object()), tcell.ColorAqua),
	)
}

func (r *GarbageRecord) String() string {
	return fmt.Sprintf(
		"CID %-44s OID %-44s",
		common.FormatSimple(fmt.Sprintf("%-44s", r.addr.Container()), tcell.ColorAqua),
		common.FormatSimple(fmt.Sprintf("%-44s", r.addr.Object()), tcell.ColorAqua),
	)
}

func (r *ContainerVolumeRecord) String() string {
	return fmt.Sprintf(
		"CID %-44s %c %d",
		common.FormatSimple(fmt.Sprintf("%-44s", r.id), tcell.ColorAqua),
		tview.Borders.Vertical,
		r.volume,
	)
}

func (r *LockedRecord) String() string {
	return fmt.Sprintf(
		"Object OID %s %c Lockers [%d]OID {...}",
		common.FormatSimple(fmt.Sprintf("%-44s", r.id), tcell.ColorAqua),
		tview.Borders.Vertical,
		len(r.ids),
	)
}

func (r *ShardInfoRecord) String() string {
	return fmt.Sprintf("%-13s %c %s", r.label, tview.Borders.Vertical, r.value)
}

func (r *ObjectRecord) String() string {
	return fmt.Sprintf(
		"OID %s %c Object {...}",
		common.FormatSimple(fmt.Sprintf("%-44s", r.id), tcell.ColorAqua),
		tview.Borders.Vertical,
	)
}

func (r *SmallRecord) String() string {
	s := fmt.Sprintf(
		"OID %s %c",
		common.FormatSimple(fmt.Sprintf("%-44s", r.id), tcell.ColorAqua),
		tview.Borders.Vertical,
	)
	if r.storageID != nil {
		s = fmt.Sprintf("%s %s", s, *r.storageID)
	}
	return s
}

func (r *RootRecord) String() string {
	s := fmt.Sprintf(
		"Root OID %s %c",
		common.FormatSimple(fmt.Sprintf("%-44s", r.id), tcell.ColorAqua),
		tview.Borders.Vertical,
	)
	if r.info != nil {
		s += " Split info {...}"
	}
	return s
}

func (r *OwnerRecord) String() string {
	return "OID " + common.FormatSimple(r.id.String(), tcell.ColorAqua)
}

func (r *UserAttributeRecord) String() string {
	return "OID " + common.FormatSimple(r.id.String(), tcell.ColorAqua)
}

func (r *PayloadHashRecord) String() string {
	return fmt.Sprintf(
		"Checksum %s %c [%d]OID {...}",
		common.FormatSimple(r.checksum.String(), tcell.ColorAqua),
		tview.Borders.Vertical,
		len(r.ids),
	)
}

func (r *ParentRecord) String() string {
	return fmt.Sprintf(
		"Parent OID %s %c [%d]OID {...}",
		common.FormatSimple(fmt.Sprintf("%-44s", r.parent), tcell.ColorAqua),
		tview.Borders.Vertical,
		len(r.ids),
	)
}

func (r *SplitRecord) String() string {
	return fmt.Sprintf(
		"Split ID %s %c [%d]OID {...}",
		common.FormatSimple(r.id.String(), tcell.ColorAqua),
		tview.Borders.Vertical,
		len(r.ids),
	)
}

func (r *ContainerCountersRecord) String() string {
	return fmt.Sprintf(
		"CID %s %c logical %d, physical %d, user %d",
		common.FormatSimple(fmt.Sprintf("%-44s", r.id), tcell.ColorAqua),
		tview.Borders.Vertical,
		r.logical, r.physical, r.user,
	)
}

func (r *ECInfoRecord) String() string {
	return fmt.Sprintf(
		"OID %s %c [%d]OID {...}",
		common.FormatSimple(fmt.Sprintf("%-44s", r.id), tcell.ColorAqua),
		tview.Borders.Vertical,
		len(r.ids),
	)
}
