package multinet

import (
	"strconv"
	"time"

	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-node/config"
)

const (
	subsection = "multinet"

	FallbackDelayDefault = 300 * time.Millisecond
)

// Enabled returns the  value of "enabled" config parameter from "multinet" section.
func Enabled(c *config.Config) bool {
	return config.BoolSafe(c.Sub(subsection), "enabled")
}

type Subnet struct {
	Mask      string
	SourceIPs []string
}

// Subnets returns the  value of "subnets" config parameter from "multinet" section.
func Subnets(c *config.Config) []Subnet {
	var result []Subnet
	sub := c.Sub(subsection).Sub("subnets")
	for i := 0; ; i++ {
		s := sub.Sub(strconv.FormatInt(int64(i), 10))
		mask := config.StringSafe(s, "mask")
		if mask == "" {
			break
		}
		sourceIPs := config.StringSliceSafe(s, "source_ips")
		result = append(result, Subnet{
			Mask:      mask,
			SourceIPs: sourceIPs,
		})
	}
	return result
}

// Balancer returns the  value of "balancer" config parameter from "multinet" section.
func Balancer(c *config.Config) string {
	return config.StringSafe(c.Sub(subsection), "balancer")
}

// Restrict returns the  value of "restrict" config parameter from "multinet" section.
func Restrict(c *config.Config) bool {
	return config.BoolSafe(c.Sub(subsection), "restrict")
}

// FallbackDelay returns the  value of "fallback_delay" config parameter from "multinet" section.
func FallbackDelay(c *config.Config) time.Duration {
	fd := config.DurationSafe(c.Sub(subsection), "fallback_delay")
	if fd != 0 { // negative value means no fallback
		return fd
	}
	return FallbackDelayDefault
}
