package tracing

import (
	"testing"

	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-node/config"
	configtest "git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-node/config/test"
	"git.frostfs.info/TrueCloudLab/frostfs-observability/tracing"
	"github.com/stretchr/testify/require"
)

func TestTracingSection(t *testing.T) {
	t.Run("defaults", func(t *testing.T) {
		tc, err := ToTracingConfig(configtest.EmptyConfig())
		require.NoError(t, err)
		require.Equal(t, false, tc.Enabled)
		require.Equal(t, tracing.Exporter(""), tc.Exporter)
		require.Equal(t, "", tc.Endpoint)
		require.Equal(t, "frostfs-node", tc.Service)
		require.Equal(t, "", tc.InstanceID)
		require.Nil(t, tc.ServerCaCertPool)
		require.Empty(t, tc.Attributes)
	})

	const path = "../../../../config/example/node"

	fileConfigTest := func(c *config.Config) {
		tc, err := ToTracingConfig(c)
		require.NoError(t, err)
		require.Equal(t, true, tc.Enabled)
		require.Equal(t, tracing.OTLPgRPCExporter, tc.Exporter)
		require.Equal(t, "localhost", tc.Endpoint)
		require.Equal(t, "frostfs-node", tc.Service)
		require.Nil(t, tc.ServerCaCertPool)
		require.EqualValues(t, map[string]string{
			"key0": "value",
			"key1": "value",
		}, tc.Attributes)
	}

	configtest.ForEachFileType(path, fileConfigTest)

	t.Run("ENV", func(t *testing.T) {
		configtest.ForEnvFileType(t, path, fileConfigTest)
	})
}
