package main

import (
	"context"
	"time"

	tracingconfig "git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-node/config/tracing"
	"git.frostfs.info/TrueCloudLab/frostfs-node/internal/logs"
	"git.frostfs.info/TrueCloudLab/frostfs-observability/tracing"
	"go.uber.org/zap"
)

func initTracing(ctx context.Context, c *cfg) {
	conf, err := tracingconfig.ToTracingConfig(c.appCfg)
	if err != nil {
		c.log.Error(ctx, logs.FrostFSNodeFailedInitTracing, zap.Error(err))
		return
	}
	_, err = tracing.Setup(ctx, *conf)
	if err != nil {
		c.log.Error(ctx, logs.FrostFSNodeFailedInitTracing, zap.Error(err))
		return
	}

	c.closers = append(c.closers, closer{
		name: "tracing",
		fn: func() {
			ctx, cancel := context.WithTimeout(context.WithoutCancel(ctx), time.Second*5)
			defer cancel()
			err := tracing.Shutdown(ctx) // cfg context cancels before close
			if err != nil {
				c.log.Error(ctx, logs.FrostFSNodeFailedShutdownTracing, zap.Error(err))
			}
		},
	})
}
